/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;
import javax.swing.AbstractListModel;
import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.dtaus.Checksum;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.IllegalHeaderException;
import org.jdtaus.banking.dtaus.IllegalTransactionException;
import org.jdtaus.banking.dtaus.LogicalFile;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.banking.dtaus.PhysicalFileException;
import org.jdtaus.banking.dtaus.PhysicalFileFactory;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.io.FileOperations;
import org.jdtaus.core.io.util.RandomAccessFileOperations;
import org.jdtaus.core.monitor.spi.Task;
import org.jdtaus.core.monitor.spi.TaskMonitor;
import org.jdtaus.core.text.Message;
import org.jdtaus.editor.Editor;
import org.jdtaus.editor.IconSet;
import org.jdtaus.editor.IllegalGeneratorTaskException;
import org.jdtaus.editor.PhysicalFileFormatter;
import org.jdtaus.editor.PhysicalFileGenerator;

/**
 * Frame to display and edit {@code PhysicalFile} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3525 2007-05-16 12:05:28Z schulte2005 $
 */
public class PhysicalFileFrame extends JFrame implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(PhysicalFileFrame.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        JFrame.setDefaultLookAndFeelDecorated(true);

        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency5;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency5 != null)
        {
            ret = this._dependency5;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency5 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency4;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency4 != null)
        {
            ret = this._dependency4;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency4 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>PhysicalFileFactory</code> implementation. */
    private transient PhysicalFileFactory _dependency3;

    /**
     * Gets the configured <code>PhysicalFileFactory</code> implementation.
     *
     * @return the configured <code>PhysicalFileFactory</code> implementation.
     */
    private PhysicalFileFactory getPhysicalFileFactory()
    {
        PhysicalFileFactory ret = null;
        if(this._dependency3 != null)
        {
            ret = this._dependency3;
        }
        else
        {
            ret = (PhysicalFileFactory) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "PhysicalFileFactory");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("PhysicalFileFactory").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Generator</code> implementation. */
    private transient PhysicalFileGenerator _dependency2;

    /**
     * Gets the configured <code>Generator</code> implementation.
     *
     * @return the configured <code>Generator</code> implementation.
     */
    private PhysicalFileGenerator getGenerator()
    {
        PhysicalFileGenerator ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (PhysicalFileGenerator) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "Generator");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("Generator").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>IconSet</code> implementation. */
    private transient IconSet _dependency1;

    /**
     * Gets the configured <code>IconSet</code> implementation.
     *
     * @return the configured <code>IconSet</code> implementation.
     */
    private IconSet getIconSet()
    {
        IconSet ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (IconSet) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "IconSet");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("IconSet").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TaskMonitor</code> implementation. */
    private transient TaskMonitor _dependency0;

    /**
     * Gets the configured <code>TaskMonitor</code> implementation.
     *
     * @return the configured <code>TaskMonitor</code> implementation.
     */
    private TaskMonitor getTaskMonitor()
    {
        TaskMonitor ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (TaskMonitor) ContainerFactory.getContainer().
                getDependency(PhysicalFileFrame.class,
                "TaskMonitor");

            if(ModelFactory.getModel().getModules().
                getImplementation(PhysicalFileFrame.class.getName()).
                getDependencies().getDependency("TaskMonitor").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--PhysicalFileFrame-------------------------------------------------------

    /** Creates new form PhysicalFileFrame */
    public PhysicalFileFrame()
    {
        super();
        this.initializeProperties(PhysicalFileFrame.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the physical file of the editor.
     *
     * @return the physical file of the editor or {@code null}.
     */
    public PhysicalFile getPhysicalFile()
    {
        return this.pFile;
    }

    /**
     * Sets the physical file to edit.
     *
     * @param pFile the physical file for the editor or {@code null}.
     */
    public void setPhysicalFile(final PhysicalFile pFile)
    {
        final Runnable runnable = new Runnable()
        {
            public void run()
            {
                firePropertyChange("physicalFile",
                    PhysicalFileFrame.this.pFile, pFile);

                PhysicalFileFrame.this.pFile = pFile;
                updatePhysicalFile();
            }
        };

        if(!SwingUtilities.isEventDispatchThread())
        {
            SwingUtilities.invokeLater(runnable);
        }
        else
        {
            runnable.run();
        }
    }

    /**
     * Flag indicating if the dialog is locked for any interaction.
     *
     * @return {@code true} if the dialog is locked for any interaction;
     * {@code false} if the dialog is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Setter for property {@code locked}.
     *
     * @param locked {@code true} if the dialog should be locked for any
     * interaction; {@code false} to unlock the dialog.
     */
    public void setLocked(final boolean locked)
    {
        final Runnable runnable = new Runnable()
        {
            public void run()
            {
                setCursor(Cursor.getPredefinedCursor(locked ?
                    Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

                logicalFileList.setEnabled(!locked);
                editMenu.setEnabled(!locked);
                fileMenu.setEnabled(!locked);
                toolsMenu.setEnabled(!locked);
                lafMenu.setEnabled(!locked);
                optionsMenu.setEnabled(!locked);
                transactionTable.setEnabled(!locked);
                PhysicalFileFrame.this.locked = locked;
            }
        };

        if(SwingUtilities.isEventDispatchThread())
        {
            SwingUtilities.invokeLater(runnable);
        }
        else
        {
            runnable.run();
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        logicalFilePanel = new javax.swing.JPanel();
        logicalFileScrollPane = new javax.swing.JScrollPane();
        logicalFileList = new javax.swing.JList();
        transactionPanel = new javax.swing.JPanel();
        transactionScrollPane = new javax.swing.JScrollPane();
        transactionTable = new javax.swing.JTable();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        newMenuItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        openMenuItem = new javax.swing.JMenuItem();
        closeMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JSeparator();
        exitMenuItem = new javax.swing.JMenuItem();
        editMenu = new javax.swing.JMenu();
        addLogicalFileMenuItem = new javax.swing.JMenuItem();
        updateLogicalFileMenuItem = new javax.swing.JMenuItem();
        removeLogicalFileMenuItem = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JSeparator();
        addTransactionMenuItem = new javax.swing.JMenuItem();
        updateTransactionMenuItem = new javax.swing.JMenuItem();
        removeTransactionMenuItem = new javax.swing.JMenuItem();
        toolsMenu = new javax.swing.JMenu();
        generatorMenuItem = new javax.swing.JMenuItem();
        bankDirectoryMenuItem = new javax.swing.JMenuItem();
        optionsMenu = new javax.swing.JMenu();
        preferencesMenuItem = new javax.swing.JMenuItem();
        lafMenu = new javax.swing.JMenu();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(PhysicalFileFrameBundle.getFrameTitleText(Locale.getDefault()));
        setIconImage(this.getIconSet().getIcon(IconSet.DEV_APPLICATION, IconSet.SIZE_16X16, Locale.getDefault()).getImage());
        setLocationByPlatform(true);
        setName("jDTAUS Editor");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                formWindowClosing(evt);
            }
        });

        logicalFilePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(PhysicalFileFrameBundle.getLogicalFilePanelTitleText(Locale.getDefault())));
        logicalFileList.setModel(new PhysicalFileFrameListModel());
        logicalFileList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        logicalFileList.setCellRenderer(new LogicalFileRenderer());
        logicalFileList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
        {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt)
            {
                logicalFileListValueChanged(evt);
            }
        });

        logicalFileScrollPane.setViewportView(logicalFileList);

        org.jdesktop.layout.GroupLayout logicalFilePanelLayout = new org.jdesktop.layout.GroupLayout(logicalFilePanel);
        logicalFilePanel.setLayout(logicalFilePanelLayout);
        logicalFilePanelLayout.setHorizontalGroup(
            logicalFilePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(logicalFileScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 753, Short.MAX_VALUE)
        );
        logicalFilePanelLayout.setVerticalGroup(
            logicalFilePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(logicalFileScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 143, Short.MAX_VALUE)
        );

        transactionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(PhysicalFileFrameBundle.getTransactionPanelTitleText(Locale.getDefault())));
        transactionTable.setModel(new PhysicalFileFrameTableModel());
        transactionTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        transactionTable.getSelectionModel().addListSelectionListener(new ListSelectionListener()
        {
            public void valueChanged(final ListSelectionEvent event)
            {
                transactionTableValueChanged(event);
            }
        });

        transactionScrollPane.setViewportView(transactionTable);

        org.jdesktop.layout.GroupLayout transactionPanelLayout = new org.jdesktop.layout.GroupLayout(transactionPanel);
        transactionPanel.setLayout(transactionPanelLayout);
        transactionPanelLayout.setHorizontalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 753, Short.MAX_VALUE)
        );
        transactionPanelLayout.setVerticalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 499, Short.MAX_VALUE)
        );

        fileMenu.setText(PhysicalFileFrameBundle.getFileMenuText(Locale.getDefault()));
        newMenuItem.setIcon(this.getIconSet().getIcon(IconSet.NEW, IconSet.SIZE_16X16, Locale.getDefault()));
        newMenuItem.setText(PhysicalFileFrameBundle.getFileNewText(Locale.getDefault()));
        newMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                newMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(newMenuItem);

        fileMenu.add(jSeparator1);

        openMenuItem.setIcon(this.getIconSet().getIcon(IconSet.OPEN, IconSet.SIZE_16X16, Locale.getDefault()));
        openMenuItem.setText(PhysicalFileFrameBundle.getFileOpenText(Locale.getDefault()));
        openMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                openMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(openMenuItem);

        closeMenuItem.setText(PhysicalFileFrameBundle.getFileCloseText(Locale.getDefault()));
        closeMenuItem.setEnabled(false);
        closeMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(closeMenuItem);

        saveAsMenuItem.setIcon(this.getIconSet().getIcon(IconSet.SAVE_AS, IconSet.SIZE_16X16, Locale.getDefault()));
        saveAsMenuItem.setText(PhysicalFileFrameBundle.getFileSaveAsText(Locale.getDefault()));
        saveAsMenuItem.setEnabled(false);
        saveAsMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                saveAsMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(saveAsMenuItem);

        fileMenu.add(jSeparator2);

        exitMenuItem.setText(PhysicalFileFrameBundle.getFileExitText(Locale.getDefault()));
        exitMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        editMenu.setText(PhysicalFileFrameBundle.getEditMenuText(Locale.getDefault()));
        addLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.ADD, IconSet.SIZE_16X16, Locale.getDefault()));
        addLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditAddLogicalFileText(Locale.getDefault()));
        addLogicalFileMenuItem.setEnabled(false);
        addLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(addLogicalFileMenuItem);

        updateLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.EDIT, IconSet.SIZE_16X16, Locale.getDefault()));
        updateLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditUpdateLogicalFileText(Locale.getDefault()));
        updateLogicalFileMenuItem.setEnabled(false);
        updateLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                updateLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(updateLogicalFileMenuItem);

        removeLogicalFileMenuItem.setIcon(this.getIconSet().getIcon(IconSet.REMOVE, IconSet.SIZE_16X16, Locale.getDefault()));
        removeLogicalFileMenuItem.setText(PhysicalFileFrameBundle.getEditRemoveLogicalFileText(Locale.getDefault()));
        removeLogicalFileMenuItem.setEnabled(false);
        removeLogicalFileMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                removeLogicalFileMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(removeLogicalFileMenuItem);

        editMenu.add(jSeparator4);

        addTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.ADD, IconSet.SIZE_16X16, Locale.getDefault()));
        addTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditAddTransactionText(Locale.getDefault()));
        addTransactionMenuItem.setEnabled(false);
        addTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(addTransactionMenuItem);

        updateTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.EDIT, IconSet.SIZE_16X16, Locale.getDefault()));
        updateTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditUpdateTransactionText(Locale.getDefault()));
        updateTransactionMenuItem.setEnabled(false);
        updateTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                updateTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(updateTransactionMenuItem);

        removeTransactionMenuItem.setIcon(this.getIconSet().getIcon(IconSet.REMOVE, IconSet.SIZE_16X16, Locale.getDefault()));
        removeTransactionMenuItem.setText(PhysicalFileFrameBundle.getEditRemoveTransactionText(Locale.getDefault()));
        removeTransactionMenuItem.setEnabled(false);
        removeTransactionMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                removeTransactionMenuItemActionPerformed(evt);
            }
        });

        editMenu.add(removeTransactionMenuItem);

        menuBar.add(editMenu);

        toolsMenu.setText(PhysicalFileFrameBundle.getToolsMenuText(Locale.getDefault()));
        generatorMenuItem.setIcon(this.getIconSet().getIcon(IconSet.NEW, IconSet.SIZE_16X16, Locale.getDefault()));
        generatorMenuItem.setText(PhysicalFileFrameBundle.getToolsGeneratorText(Locale.getDefault()));
        generatorMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                generatorMenuItemActionPerformed(evt);
            }
        });

        toolsMenu.add(generatorMenuItem);

        bankDirectoryMenuItem.setAction(new SearchBankleitzahlAction(null, false));
        toolsMenu.add(bankDirectoryMenuItem);

        menuBar.add(toolsMenu);

        optionsMenu.setText(PhysicalFileFrameBundle.getOptionsMenuText(Locale.getDefault()));
        preferencesMenuItem.setIcon(this.getIconSet().getIcon(IconSet.PREFERENCES, IconSet.SIZE_16X16, Locale.getDefault()));
        preferencesMenuItem.setText(PhysicalFileFrameBundle.getOptionsPreferencesText(Locale.getDefault()));
        preferencesMenuItem.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                preferencesMenuItemActionPerformed(evt);
            }
        });

        optionsMenu.add(preferencesMenuItem);

        this.initLafMenu();
        lafMenu.setText(PhysicalFileFrameBundle.getLafMenuText(Locale.getDefault()));
        optionsMenu.add(lafMenu);

        menuBar.add(optionsMenu);

        setJMenuBar(menuBar);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, logicalFilePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(logicalFilePanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void preferencesMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_preferencesMenuItemActionPerformed
    {//GEN-HEADEREND:event_preferencesMenuItemActionPerformed
        final ConfigurationDialog dialog = this.getUserInterface().
            getConfigurationDialog();

        dialog.setPreferences(this.getApplication().getPreferences());

        this.getUserInterface().push(dialog);
        dialog.setVisible(true);
        this.getUserInterface().pop();
    }//GEN-LAST:event_preferencesMenuItemActionPerformed

    private void saveAsMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_saveAsMenuItemActionPerformed
    {//GEN-HEADEREND:event_saveAsMenuItemActionPerformed
        final JFileChooser chooser = this.getUserInterface().
            getSaveAsFileChooser();

        final int ret = chooser.showSaveDialog(
            this.getUserInterface().getParent());

        if(ret == JFileChooser.APPROVE_OPTION &&
            chooser.getSelectedFile() != null)
        {
            final File file = chooser.getSelectedFile();
            final PhysicalFileFormatter formatter =
                ((PhysicalFileFormatterFileFilter) chooser.
                getFileFilter()).getFormatter();

            boolean doOverwrite = true;

            if(!this.getApplication().getPreferences().
                isOverwriteConfirmationDisabled() && file.exists())
            {
                doOverwrite = JOptionPane.showConfirmDialog(
                    this.getUserInterface().getParent(),
                    PhysicalFileFrameBundle.getOverwriteConfirmationText(
                    Locale.getDefault())) == JOptionPane.OK_OPTION;

            }

            if(doOverwrite)
            {
                final Thread t = new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            setLocked(true);
                            formatter.format(pFile, file);
                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        finally
                        {
                            setLocked(false);
                        }
                    }
                };

                t.setUncaughtExceptionHandler(this.getExceptionHandler());
                t.start();
            }
        }
    }//GEN-LAST:event_saveAsMenuItemActionPerformed

    private void closeMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeMenuItemActionPerformed
    {//GEN-HEADEREND:event_closeMenuItemActionPerformed
        this.setPhysicalFile(null);
    }//GEN-LAST:event_closeMenuItemActionPerformed

    private void generatorMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_generatorMenuItemActionPerformed
    {//GEN-HEADEREND:event_generatorMenuItemActionPerformed
        final GeneratorDialog dialog = this.getUserInterface().
            getGeneratorDialog();

        dialog.setGeneratorTask(dialog.getGeneratorTask());
        dialog.onIllegalGeneratorTask(null);
        dialog.setActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e)
            {
                if(dialog.getOutputFile() != null)
                {
                    boolean doOverwrite = true;
                    if(!getApplication().getPreferences().
                        isOverwriteConfirmationDisabled() &&
                        dialog.getOutputFile().exists())
                    {
                        doOverwrite = JOptionPane.showConfirmDialog(
                            getUserInterface().getParent(),
                            GeneratorDialogBundle.
                            getOverwriteConfirmationText(
                            Locale.getDefault())) == JOptionPane.OK_OPTION;

                    }

                    if(doOverwrite)
                    {
                        final Thread t = new Thread()
                        {
                            public void run()
                            {
                                try
                                {
                                    dialog.setLocked(true);
                                    setPhysicalFile(getGenerator().generate(
                                        dialog.getOutputFile(),
                                        dialog.getGeneratorTask()));

                                    SwingUtilities.invokeLater(new Runnable()
                                    {
                                        public void run()
                                        {
                                            dialog.setVisible(false);
                                            dialog.dispose();
                                        }
                                    });
                                }
                                catch(final IllegalGeneratorTaskException e)
                                {
                                    UIManager.getLookAndFeel().
                                        provideErrorFeedback(getUserInterface().
                                        getParent());

                                    dialog.onIllegalGeneratorTask(e);
                                }
                                catch(IOException e)
                                {
                                    onError(e, e.getMessage());
                                }
                                finally
                                {
                                    dialog.setLocked(false);
                                }
                            }
                        };

                        t.setUncaughtExceptionHandler(
                            getExceptionHandler());

                        t.start();
                    }
                }
                else
                {
                    JOptionPane.showMessageDialog(getUserInterface().
                        getParent(), PhysicalFileFrameBundle.
                        getNoOutputFileText(Locale.getDefault()));

                }
            }
        });

        this.getUserInterface().push(dialog);
        dialog.setVisible(true);
        this.getUserInterface().pop();
    }//GEN-LAST:event_generatorMenuItemActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt)//GEN-FIRST:event_formWindowClosing
    {//GEN-HEADEREND:event_formWindowClosing
        if(!this.isLocked())
        {
            this.setVisible(false);
            this.dispose();
        }
    }//GEN-LAST:event_formWindowClosing

    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_exitMenuItemActionPerformed
    {//GEN-HEADEREND:event_exitMenuItemActionPerformed
        this.setVisible(false);
        this.dispose();
    }//GEN-LAST:event_exitMenuItemActionPerformed

    private void removeTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_removeTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_removeTransactionMenuItemActionPerformed
        boolean doDelete = true;

        if(!this.getApplication().getPreferences().
            isPaymentDeletionConfirmationDisabled())
        {
            doDelete = JOptionPane.showConfirmDialog(
                this.getUserInterface().getParent(), PhysicalFileFrameBundle.
                getPaymentDeletionConfirmationText(
                Locale.getDefault())) == JOptionPane.OK_OPTION;

        }

        if(doDelete)
        {
            final Thread t = new Thread()
            {
                public void run()
                {
                    try
                    {
                        setLocked(true);
                        lFile.removeTransaction(selectedTransaction.getIndex());
                        selectedTransaction = null;
                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    finally
                    {
                        setLocked(false);
                        updateLogicalFile();
                    }
                }
            };

            t.setUncaughtExceptionHandler(this.getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_removeTransactionMenuItemActionPerformed

    private void removeLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_removeLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_removeLogicalFileMenuItemActionPerformed
        boolean doDelete = true;

        if(!this.getApplication().getPreferences().
            isFileDeletionConfirmationDisabled())
        {
            doDelete = JOptionPane.showConfirmDialog(
                this.getUserInterface().getParent(), PhysicalFileFrameBundle.
                getHeaderDeletionConfirmationText(
                Locale.getDefault())) == JOptionPane.OK_OPTION;

        }

        if(doDelete)
        {
            final Thread t = new Thread()
            {
                public void run()
                {
                    try
                    {
                        setLocked(true);
                        pFile.remove(lFile.getIndex());
                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    finally
                    {
                        setLocked(false);
                        lFile = null;
                        updateLogicalFile();
                        updatePhysicalFile();
                    }
                }
            };

            t.setUncaughtExceptionHandler(this.getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_removeLogicalFileMenuItemActionPerformed

    private void addTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_addTransactionMenuItemActionPerformed
        try
        {
            final TransactionDialog dialog = this.getUserInterface().
                getTransactionDialog();

            dialog.setDebit(lFile.getHeader().getType().isDebitAllowed());
            dialog.setRemittance(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setDate(lFile.getHeader().getCreateDate());
            dialog.setTransaction(null);
            dialog.onIllegalTransaction(null);
            dialog.setActionListener(new ActionListener()
            {
                public void actionPerformed(final ActionEvent e)
                {
                    final Thread th = new Thread()
                    {
                        public void run()
                        {
                            try
                            {
                                dialog.setLocked(true);
                                lFile.createTransaction(
                                    dialog.getTransaction());

                                final int index = lFile.getChecksum().
                                    getTransactionCount() - 1;

                                selectedTransaction =
                                    new TransactionModel(index,
                                    lFile.getTransaction(index));

                                updateLogicalFile();

                                SwingUtilities.invokeLater(new Runnable()
                                {
                                    public void run()
                                    {
                                        dialog.setVisible(false);
                                        dialog.dispose();
                                    }
                                });
                            }
                            catch(final IllegalTransactionException e)
                            {
                                UIManager.getLookAndFeel().provideErrorFeedback(
                                    getUserInterface().getParent());

                                dialog.onIllegalTransaction(e);
                            }
                            catch(IOException e)
                            {
                                onError(e, e.getMessage());
                            }
                            finally
                            {
                                dialog.setLocked(false);
                            }
                        }
                    };

                    th.setUncaughtExceptionHandler(getExceptionHandler());
                    th.start();
                }
            });

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_addTransactionMenuItemActionPerformed

    private void updateLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_updateLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_updateLogicalFileMenuItemActionPerformed
        try
        {
            final HeaderDialog dialog = this.getUserInterface().
                getHeaderDialog();

            dialog.setDebitSupported(lFile.getHeader().getType().
                isDebitAllowed());

            dialog.setRemittanceSupported(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setHeader(lFile.getHeader());
            dialog.onIllegalHeader(null);
            dialog.setActionListener(new ActionListener()
            {
                public void actionPerformed(final ActionEvent e)
                {
                    final Thread t = new Thread()
                    {
                        public void run()
                        {
                            try
                            {
                                dialog.setLocked(true);
                                lFile.setHeader(dialog.getHeader());

                                SwingUtilities.invokeLater(new Runnable()
                                {
                                    public void run()
                                    {
                                        headerUpdating = true;
                                        getListModel().sort();
                                        logicalFileList.setSelectedValue(
                                            lFile, true);

                                        headerUpdating = false;
                                        dialog.setVisible(false);
                                        dialog.dispose();
                                    }
                                });
                            }
                            catch(final IllegalHeaderException e)
                            {
                                UIManager.getLookAndFeel().provideErrorFeedback(
                                    getUserInterface().getParent());

                                dialog.onIllegalHeader(e);
                            }
                            catch(IOException e)
                            {
                                onError(e, e.getMessage());
                            }
                            finally
                            {
                                dialog.setLocked(false);
                            }
                        }
                    };

                    t.setUncaughtExceptionHandler(getExceptionHandler());
                    t.start();
                }
            });

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_updateLogicalFileMenuItemActionPerformed

    private void addLogicalFileMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_addLogicalFileMenuItemActionPerformed
    {//GEN-HEADEREND:event_addLogicalFileMenuItemActionPerformed
        final HeaderDialog dialog = this.getUserInterface().getHeaderDialog();

        dialog.setDebitSupported(true);
        dialog.setRemittanceSupported(true);
        dialog.setHeader(null);
        dialog.onIllegalHeader(null);
        dialog.setActionListener(new ActionListener()
        {
            public void actionPerformed(final ActionEvent e)
            {
                final Thread t = new Thread()
                {
                    public void run()
                    {
                        try
                        {
                            dialog.setLocked(true);
                            lFile = new LogicalFileModel(pFile.count(),
                                pFile.add(dialog.getHeader()));

                            updateLogicalFile();
                            updatePhysicalFile();

                            SwingUtilities.invokeLater(new Runnable()
                            {
                                public void run()
                                {
                                    dialog.setVisible(false);
                                    dialog.dispose();
                                }
                            });
                        }
                        catch(final IllegalHeaderException e)
                        {
                            UIManager.getLookAndFeel().provideErrorFeedback(
                                getUserInterface().getParent());

                            dialog.onIllegalHeader(e);
                        }
                        catch(IOException e)
                        {
                            onError(e, e.getMessage());
                        }
                        finally
                        {
                            dialog.setLocked(false);
                        }
                    }
                };

                t.setUncaughtExceptionHandler(getExceptionHandler());
                t.start();
            }
        });

        this.getUserInterface().push(dialog);
        dialog.setVisible(true);
        this.getUserInterface().pop();
    }//GEN-LAST:event_addLogicalFileMenuItemActionPerformed

    private void newMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_newMenuItemActionPerformed
    {//GEN-HEADEREND:event_newMenuItemActionPerformed
        final int option;
        final JFileChooser chooser = this.getUserInterface().
            getOpenFileChooser();

        option = chooser.showSaveDialog(this.getUserInterface().getParent());

        if(option == JFileChooser.APPROVE_OPTION)
        {
            final File file = chooser.getSelectedFile();

            if(file != null)
            {
                boolean doOverwrite = true;

                if(!this.getApplication().getPreferences().
                    isOverwriteConfirmationDisabled() && file.exists())
                {
                    doOverwrite = JOptionPane.showConfirmDialog(
                        this.getUserInterface().getParent(),
                        PhysicalFileFrameBundle.getOverwriteConfirmationText(
                        Locale.getDefault())) == JOptionPane.OK_OPTION;

                }

                if(doOverwrite)
                {
                    final Thread t = new Thread()
                    {
                        public void run()
                        {
                            PhysicalFile physFile = null;

                            try
                            {
                                setLocked(true);

                                final FileOperations ops =
                                    new RandomAccessFileOperations(
                                    new RandomAccessFile(file, "rw"));

                                physFile = getPhysicalFileFactory().
                                    createPhysicalFile(ops,
                                    PhysicalFileFactory.FORMAT_DISK);

                            }
                            catch(IOException e)
                            {
                                onError(e, e.getMessage());
                            }
                            finally
                            {
                                setLocked(true);
                                setPhysicalFile(physFile);
                            }
                        }
                    };

                    t.setUncaughtExceptionHandler(getExceptionHandler());
                    t.start();
                }
            }
        }
    }//GEN-LAST:event_newMenuItemActionPerformed

    private void updateTransactionMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_updateTransactionMenuItemActionPerformed
    {//GEN-HEADEREND:event_updateTransactionMenuItemActionPerformed
        try
        {
            final int modelIndex = this.getTableModel().getEntities().
                indexOf(this.selectedTransaction);

            final TransactionDialog dialog = this.getUserInterface().
                getTransactionDialog();

            dialog.setDebit(lFile.getHeader().getType().
                isDebitAllowed());

            dialog.setRemittance(lFile.getHeader().getType().
                isRemittanceAllowed());

            dialog.setDate(lFile.getHeader().getCreateDate());
            dialog.setTransaction(this.selectedTransaction);
            dialog.onIllegalTransaction(null);
            dialog.setActionListener(new ActionListener()
            {
                public void actionPerformed(final ActionEvent e)
                {
                    final Thread t = new Thread()
                    {
                        public void run()
                        {
                            try
                            {
                                dialog.setLocked(true);

                                final int index =
                                    selectedTransaction.getIndex();

                                lFile.setTransaction(index,
                                    dialog.getTransaction());

                                selectedTransaction =
                                    new TransactionModel(index,
                                    lFile.getTransaction(index));

                                SwingUtilities.invokeLater(new Runnable()
                                {
                                    public void run()
                                    {
                                        getTableModel().getEntities().
                                            set(modelIndex,
                                            selectedTransaction);

                                        // Save the selection since it is
                                        // overwritten by sort().
                                        final TransactionModel selection =
                                            selectedTransaction;

                                        getTableModel().sort();
                                        final int newModelIndex =
                                            getTableModel().getEntities().
                                            indexOf(selection);

                                        transactionTable.
                                            getSelectionModel().
                                            setSelectionInterval(
                                            newModelIndex,
                                            newModelIndex);

                                        transactionTable.
                                            scrollRectToVisible(
                                            transactionTable.getCellRect(
                                            newModelIndex, 0, true));

                                        dialog.setVisible(false);
                                        dialog.dispose();
                                    }
                                });
                            }
                            catch(final IllegalTransactionException e)
                            {
                                UIManager.getLookAndFeel().provideErrorFeedback(
                                    getUserInterface().getParent());

                                dialog.onIllegalTransaction(e);
                            }
                            catch(IOException e)
                            {
                                onError(e, e.getMessage());
                            }
                            finally
                            {
                                dialog.setLocked(false);
                            }
                        }
                    };

                    t.setUncaughtExceptionHandler(getExceptionHandler());
                    t.start();
                }
            });

            getUserInterface().push(dialog);
            dialog.setVisible(true);
            getUserInterface().pop();
        }
        catch(IOException e)
        {
            onError(e, e.getMessage());
        }
    }//GEN-LAST:event_updateTransactionMenuItemActionPerformed

    private void logicalFileListValueChanged(javax.swing.event.ListSelectionEvent evt)//GEN-FIRST:event_logicalFileListValueChanged
    {//GEN-HEADEREND:event_logicalFileListValueChanged
        if(!evt.getValueIsAdjusting() && !this.headerUpdating)
        {
            this.lFile = (LogicalFileModel) this.logicalFileList.
                getSelectedValue();

            this.updateLogicalFile();
        }
    }//GEN-LAST:event_logicalFileListValueChanged

    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_openMenuItemActionPerformed
    {//GEN-HEADEREND:event_openMenuItemActionPerformed
        final File file;
        final int option;
        final JFileChooser chooser = this.getUserInterface().
            getOpenFileChooser();

        option = chooser.showOpenDialog(this.getUserInterface().getParent());
        file = chooser.getSelectedFile();

        if(file != null && option == JFileChooser.APPROVE_OPTION)
        {
            final Thread t = new Thread()
            {
                public void run()
                {
                    PhysicalFile physFile = null;

                    try
                    {
                        setLocked(true);
                        physFile = getPhysicalFileFactory().getPhysicalFile(
                            new RandomAccessFileOperations(
                            new RandomAccessFile(file, "rw")));

                    }
                    catch(IOException e)
                    {
                        onError(e, e.getMessage());
                    }
                    catch(PhysicalFileException e)
                    {
                        final Object[] messages =
                            new Object[e.getMessages().length];

                        for(int i = 0; i < messages.length && i < 25; i++)
                        {
                            messages[i] = e.getMessages()[i].
                                getText(Locale.getDefault());

                        }

                        onError(e, messages);
                    }
                    finally
                    {
                        setLocked(false);
                        setPhysicalFile(physFile);
                    }
                }
            };

            t.setUncaughtExceptionHandler(getExceptionHandler());
            t.start();
        }
    }//GEN-LAST:event_openMenuItemActionPerformed

    private void transactionTableValueChanged(final ListSelectionEvent evt)
    {
        if(!evt.getValueIsAdjusting())
        {
            if(evt.getFirstIndex() < 0)
            {
                this.selectedTransaction = null;
                this.updateTransactionMenuItem.setEnabled(false);
                this.removeTransactionMenuItem.setEnabled(false);
            }
            else
            {
                this.selectedTransaction = getTableModel().getEntities().
                    get(transactionTable.getSelectedRow());

                this.updateTransactionMenuItem.setEnabled(true);
                this.removeTransactionMenuItem.setEnabled(true);
            }
        }
    }

    private void onError(final Throwable t, final Object messages)
    {
        UIManager.getLookAndFeel().provideErrorFeedback(
            getUserInterface().getParent());

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final String name = t.getClass().getName();
                JOptionPane.showMessageDialog(getUserInterface().getParent(),
                    messages, name.substring(name.lastIndexOf('.') + 1),
                    JOptionPane.ERROR_MESSAGE);

            }
        });
    }

    private void updatePhysicalFile()
    {
        final Thread t = new Thread()
        {
            public void run()
            {
                final LogicalFile prevSelection = lFile;

                try
                {
                    setLocked(true);

                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            addLogicalFileMenuItem.setEnabled(pFile != null);
                            closeMenuItem.setEnabled(pFile != null);
                            saveAsMenuItem.setEnabled(pFile != null);
                            getTableModel().clear();
                            getListModel().clear();
                        }
                    });

                    if(pFile != null)
                    {
                        final LogicalFileModel lFile;
                        final Collection<LogicalFileModel> files =
                            new ArrayList(pFile.count());

                        for(int i = 0; i < pFile.count(); i++)
                        {
                            files.add(new LogicalFileModel(i, pFile.get(i)));
                        }

                        SwingUtilities.invokeLater(new Runnable()
                        {
                            public void run()
                            {
                                getListModel().addAll(files);
                            }
                        });
                    }
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                finally
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            logicalFileList.setSelectedValue(
                                prevSelection, true);

                        }
                    });

                    setLocked(false);
                }
            }
        };

        t.setUncaughtExceptionHandler(getExceptionHandler());
        t.start();
    }

    private void updateLogicalFile()
    {
        final Thread t = new Thread()
        {
            public void run()
            {
                final TransactionModel prevSelection = selectedTransaction;
                final Task task = new Task();

                try
                {
                    setLocked(true);

                    SwingUtilities.invokeAndWait(new Runnable()
                    {
                        public void run()
                        {
                            getTableModel().clear();
                        }
                    });

                    if(lFile != null)
                    {
                        final Collection<TransactionModel> col =
                            new ArrayList(lFile.getChecksum().
                            getTransactionCount());

                        final int count = lFile.getChecksum().
                            getTransactionCount();

                        task.setIndeterminate(false);
                        task.setCancelable(false);
                        task.setMinimum(0);
                        task.setMaximum(count);
                        task.setProgress(0);
                        task.setDescription(new Message()
                        {
                            public Object[] getFormatArguments(Locale locale)
                            {
                                return new Object[0];
                            }
                            public String getText(final Locale locale)
                            {
                                return PhysicalFileFrameBundle.
                                    getSortingTransactionsText(locale);

                            }
                        });

                        getTaskMonitor().monitor(task);

                        for(int i = 0; i < count; i++)
                        {
                            col.add(new TransactionModel(i,
                                lFile.getTransaction(i)));

                            task.setProgress(i);
                        }

                        SwingUtilities.invokeAndWait(new Runnable()
                        {
                            public void run()
                            {
                                getTableModel().addAll(col);
                            }
                        });
                    }
                }
                catch(IOException e)
                {
                    onError(e, e.getMessage());
                }
                catch (InvocationTargetException ex)
                {
                    throw new AssertionError(ex);
                }
                catch (InterruptedException ex)
                {
                    throw new AssertionError(ex);
                }
                finally
                {
                    getTaskMonitor().finish(task);
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            updateLogicalFileMenuItem.setEnabled(lFile != null);
                            removeLogicalFileMenuItem.setEnabled(lFile != null);
                            addTransactionMenuItem.setEnabled(lFile != null);

                            if(prevSelection != null)
                            {
                                final int index = getTableModel().getEntities().
                                    indexOf(prevSelection);

                                transactionTable.getSelectionModel().
                                    setSelectionInterval(index, index);

                                transactionTable.scrollRectToVisible(
                                    transactionTable.getCellRect(
                                    index, 0, true));

                            }
                            else
                            {
                                transactionTable.getSelectionModel().
                                    clearSelection();

                            }
                        }
                    });

                    setLocked(false);
                }
            }
        };

        t.setUncaughtExceptionHandler(getExceptionHandler());
        t.start();
    }

    private void initLafMenu()
    {
        final ButtonGroup buttonGroup = new ButtonGroup();
        final UIManager.LookAndFeelInfo[] info =
            UIManager.getInstalledLookAndFeels();

        Arrays.sort(info, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof UIManager.LookAndFeelInfo;
                assert o2 instanceof UIManager.LookAndFeelInfo;

                return ((UIManager.LookAndFeelInfo) o1).getName().
                    compareTo(((UIManager.LookAndFeelInfo) o2).getName());

            }
        });

        for(int i = 0; i < info.length; i++)
        {
            final String lafClass = info[i].getClassName();
            final JRadioButtonMenuItem item = new JRadioButtonMenuItem();

            buttonGroup.add(item);

            item.setText(info[i].getName());
            if(UIManager.getSystemLookAndFeelClassName().
                equals(info[i].getClassName()))
            {
                item.setSelected(true);
            }

            item.addActionListener(new ActionListener()
            {
                public void actionPerformed(final ActionEvent e)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            try
                            {
                                UIManager.setLookAndFeel(lafClass);
                                getUserInterface().updateComponentTreeUI();
                            }
                            catch(ClassNotFoundException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(InstantiationException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(IllegalAccessException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                            catch(UnsupportedLookAndFeelException ex)
                            {
                                onError(ex, ex.getMessage());
                            }
                        }
                    });
                }
            });

            this.lafMenu.add(item);
        }
    }

    private PhysicalFileFrameTableModel getTableModel()
    {
        return (PhysicalFileFrameTableModel) this.transactionTable.getModel();
    }

    private PhysicalFileFrameListModel getListModel()
    {
        return (PhysicalFileFrameListModel) this.logicalFileList.getModel();
    }

    private Thread.UncaughtExceptionHandler getExceptionHandler()
    {
        return this.getApplication().getExceptionHandler();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem addLogicalFileMenuItem;
    private javax.swing.JMenuItem addTransactionMenuItem;
    private javax.swing.JMenuItem bankDirectoryMenuItem;
    private javax.swing.JMenuItem closeMenuItem;
    private javax.swing.JMenu editMenu;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenuItem generatorMenuItem;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JMenu lafMenu;
    private javax.swing.JList logicalFileList;
    private javax.swing.JPanel logicalFilePanel;
    private javax.swing.JScrollPane logicalFileScrollPane;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem newMenuItem;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JMenu optionsMenu;
    private javax.swing.JMenuItem preferencesMenuItem;
    private javax.swing.JMenuItem removeLogicalFileMenuItem;
    private javax.swing.JMenuItem removeTransactionMenuItem;
    private javax.swing.JMenuItem saveAsMenuItem;
    private javax.swing.JMenu toolsMenu;
    private javax.swing.JPanel transactionPanel;
    private javax.swing.JScrollPane transactionScrollPane;
    private javax.swing.JTable transactionTable;
    private javax.swing.JMenuItem updateLogicalFileMenuItem;
    private javax.swing.JMenuItem updateTransactionMenuItem;
    // End of variables declaration//GEN-END:variables

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    /** Physical file currently being edited or {@code null}. */
    private transient PhysicalFile pFile;

    /** Logical file currently selected or {@code null}. */
    private transient LogicalFileModel lFile;

    /** Currently selected transaction or {@code null}. */
    private transient TransactionModel selectedTransaction;

    /** Flag indicating that the header of {@code lFile} is updating. */
    private transient boolean headerUpdating;

    //-------------------------------------------------------PhysicalFileFrame--

}

/**
 * Model for the table holding {@code TransactionView} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3525 2007-05-16 12:05:28Z schulte2005 $
 */
class PhysicalFileFrameTableModel implements TableModel
{
    //--TableModel--------------------------------------------------------------

    /**
     * {@code TableModelEvent} listener list.
     * @serial
     */
    private EventListenerList tableListeners = new EventListenerList();

    public int getRowCount()
    {
        return this.getEntities().size();
    }

    public int getColumnCount()
    {
        return 8;
    }

    public String getColumnName(int columnIndex)
    {
        String name;

        switch(columnIndex)
        {
            case 0: name = PhysicalFileFrameBundle.
                getTransactionExecutiveNameText(Locale.getDefault());
            break;

            case 1: name = PhysicalFileFrameBundle.
                getTransactionExecutiveAccountText(Locale.getDefault());
            break;

            case 2: name = PhysicalFileFrameBundle.
                getTransactionExecutiveBankText(Locale.getDefault());
            break;

            case 3: name = PhysicalFileFrameBundle.
                getTransactionTargetNameText(Locale.getDefault());
            break;

            case 4: name = PhysicalFileFrameBundle.
                getTransactionTargetAccountText(Locale.getDefault());
            break;

            case 5: name = PhysicalFileFrameBundle.
                getTransactionTargetBankText(Locale.getDefault());
            break;

            case 6: name = PhysicalFileFrameBundle.
                getTransactionReferenceText(Locale.getDefault());
            break;

            case 7: name = PhysicalFileFrameBundle.
                getTransactionAmountText(Locale.getDefault());
            break;

            default: name = "";

        }

        return name;
    }

    public Class<?> getColumnClass(int columnIndex)
    {
        return String.class;
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return false;
    }

    public Object getValueAt(int rowIndex, int columnIndex)
    {
        if(rowIndex >= this.getEntities().size())
        {
            throw new IllegalArgumentException(Integer.toString(rowIndex));
        }
        if(columnIndex >= this.getColumnCount())
        {
            throw new IllegalArgumentException(Integer.toString(columnIndex));
        }

        String value;
        final Transaction t = this.getEntities().get(rowIndex);
        switch(columnIndex)
        {
            case 0: value = t.getExecutiveName() == null ?
                "" : t.getExecutiveName().format();
            break;

            case 1: value = t.getExecutiveAccount() == null ?
                "" : t.getExecutiveAccount().format(
                Kontonummer.LETTER_FORMAT);
            break;

            case 2: value = t.getExecutiveBank() == null ?
                "" : t.getExecutiveBank().format(
                Bankleitzahl.LETTER_FORMAT);
            break;

            case 3: value = t.getTargetName() == null ?
                "" : t.getTargetName().format();
            break;

            case 4: value = t.getTargetAccount() == null ?
                "" : t.getTargetAccount().format(
                Kontonummer.LETTER_FORMAT);
            break;

            case 5: value = t.getTargetBank() == null ?
                "" : t.getTargetBank().format(
                Bankleitzahl.LETTER_FORMAT);
            break;

            case 6: value = t.getReference() == null ? ""
                : t.getReference().format(Referenznummer11.LETTER_FORMAT);
            break;

            case 7:
                if(t.getAmount() != null && t.getCurrency() != null)
                {
                    final NumberFormat fmt = NumberFormat.getCurrencyInstance();
                    final int fractionDigits = t.getCurrency().
                        getDefaultFractionDigits();

                    fmt.setMinimumFractionDigits(fractionDigits);
                    fmt.setMaximumFractionDigits(fractionDigits);
                    fmt.setCurrency(t.getCurrency());

                    final BigDecimal amount = new BigDecimal(t.getAmount()).
                        divide(new BigDecimal(Math.pow(10, fractionDigits)),
                        fractionDigits, RoundingMode.HALF_UP);

                    value = fmt.format(amount);
                }
                else
                {
                    value = "";
                }
                break;

            default: value = t.toString();
        }

        return value;
    }

    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
    }

    public final void addTableModelListener(final TableModelListener l)
    {
        this.tableListeners.add(TableModelListener.class, l);
    }

    public final void removeTableModelListener(final TableModelListener l)
    {
        this.tableListeners.remove(TableModelListener.class, l);
    }

    //--------------------------------------------------------------TableModel--
    //--TableModel--------------------------------------------------------------

    /**
     * Collection of {@code Transaction} instances.
     * @serial
     */
    private List<TransactionModel> entities;

    /** Sorts {@code Transaction} instances. */
    private static Comparator TRANSACTION_SORTER = new Comparator()
    {
        public int compare(final Object o1, final Object o2)
        {
            if(!(o1 instanceof Transaction))
            {
                throw new IllegalArgumentException(o1.getClass().getName());
            }
            if(!(o2 instanceof Transaction))
            {
                throw new IllegalArgumentException(o2.getClass().getName());
            }

            int ret;
            final Transaction t1 = (Transaction) o1;
            final Transaction t2 = (Transaction) o2;

            if(t1.getExecutiveName() == null)
            {
                ret = t2.getExecutiveName() == null ? 0 : -1;
            }
            else
            {
                ret = t1.getExecutiveName().compareTo(t2.getExecutiveName());
            }

            if(ret == 0)
            {
                if(t1.getExecutiveAccount() == null)
                {
                    ret = t2.getExecutiveAccount() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getExecutiveAccount().compareTo(
                        t2.getExecutiveAccount());

                }
            }

            if(ret == 0)
            {
                if(t1.getExecutiveBank() == null)
                {
                    ret = t2.getExecutiveBank() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getExecutiveBank().compareTo(
                        t2.getExecutiveBank());

                }
            }

            if(ret == 0)
            {
                if(t1.getTargetName() == null)
                {
                    ret = t2.getTargetName() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetName().compareTo(t2.getTargetName());
                }
            }

            if(ret == 0)
            {
                if(t1.getTargetAccount() == null)
                {
                    ret = t2.getTargetAccount() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetAccount().compareTo(
                        t2.getTargetAccount());

                }
            }

            if(ret == 0)
            {
                if(t1.getTargetBank() == null)
                {
                    ret = t2.getTargetBank() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getTargetBank().compareTo(t2.getTargetBank());
                }
            }

            if(ret == 0)
            {
                if(t1.getReference() == null)
                {
                    ret = t2.getReference() == null ? 0 : -1;
                }
                else
                {
                    ret = t1.getReference().compareTo(t2.getReference());
                }
            }

            return ret;
        }
    };

    /**
     * Gets the entities of the model.
     *
     * @return all entities of the model.
     */
    public List<TransactionModel> getEntities()
    {
        if(this.entities == null)
        {
            this.entities = Collections.synchronizedList(new ArrayList(50));
        }

        return this.entities;
    }

    protected void add(final TransactionModel transaction)
    {
        if(transaction == null)
        {
            throw new NullPointerException("transaction");
        }

        this.getEntities().add(transaction);
        Collections.sort(this.getEntities(), TRANSACTION_SORTER);
        this.fireTableChanged(new TableModelEvent(this));
    }

    protected void addAll(final Collection<TransactionModel> transactions)
    {
        if(transactions != null)
        {
            this.getEntities().addAll(transactions);
        }
        Collections.sort(this.getEntities(), TRANSACTION_SORTER);
        this.fireTableChanged(new TableModelEvent(this));
    }

    protected void remove(final Transaction transaction)
    {
        if(transaction == null)
        {
            throw new NullPointerException("transaction");
        }

        final int row = this.getEntities().indexOf(transaction);
        if(row != -1)
        {
            this.getEntities().remove(transaction);
            this.fireTableChanged(new TableModelEvent(this, row, row,
                TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));

        }
    }

    protected void clear()
    {
        this.getEntities().clear();
        this.fireTableChanged(new TableModelEvent(this));
    }

    protected void sort()
    {
        Collections.sort(this.getEntities(), TRANSACTION_SORTER);
        this.fireTableChanged(new TableModelEvent(this));
    }

    /**
     * Notifies all registered {@code TableModelListener}s about {@code event}.
     *
     * @param event the event for the listeners.
     */
    protected void fireTableChanged(final TableModelEvent event)
    {
        if(event != null)
        {
            final Object[] listeners = this.tableListeners.getListenerList();
            for (int i = listeners.length - 2; i >= 0; i -= 2)
            {
                if (listeners[i] == TableModelListener.class)
                {
                    ((TableModelListener) listeners[i + 1]).tableChanged(event);
                }
            }
        }
    }

    //--------------------------------------------------------------TableModel--

}

/**
 * Model for the list holding {@code LogicalFile} instances.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3525 2007-05-16 12:05:28Z schulte2005 $
 */
class PhysicalFileFrameListModel extends AbstractListModel
{
    //--AbstractListModel-------------------------------------------------------

    public final int getSize()
    {
        return this.getEntities().size();
    }

    public final Object getElementAt(int index)
    {
        return this.getEntities().get(index);
    }

    //-------------------------------------------------------AbstractListModel--
    //--PhysicalFileFrameListModel----------------------------------------------

    /** Sorts {@code LogicalFile} instances. */
    private final Comparator LOGICALFILE_SORTER = new Comparator()
    {
        public int compare(final Object o1, final Object o2)
        {
            try
            {
                if(!(o1 instanceof LogicalFile))
                {
                    throw new ClassCastException(o1.getClass().getName());
                }
                if(!(o2 instanceof LogicalFile))
                {
                    throw new ClassCastException(o2.getClass().getName());
                }

                int ret;
                final LogicalFile l1 = (LogicalFile) o1;
                final LogicalFile l2 = (LogicalFile) o2;

                if(l1.getHeader().getReference() == null)
                {
                    ret  = l2.getHeader().getReference() == null ? 0 : -1;
                }
                else
                {
                    ret = l1.getHeader().getReference().
                        compareTo(l2.getHeader().getReference());

                }

                if(ret == 0)
                {
                    if(l1.getHeader().getCustomer() == null)
                    {
                        ret = l2.getHeader().getCustomer() == null ? 0 : -1;
                    }
                    else
                    {
                        ret = l1.getHeader().getCustomer().
                            compareTo(l2.getHeader().getCustomer());

                    }
                }

                if(ret == 0)
                {
                    if(l1.getHeader().getAccount() == null)
                    {
                        ret = l2.getHeader().getAccount() == null ? 0 : -1;
                    }
                    else
                    {
                        ret = l1.getHeader().getAccount().
                            compareTo(l2.getHeader().getAccount());

                    }
                }

                if(ret == 0)
                {
                    if(l1.getHeader().getBank() == null)
                    {
                        ret = l2.getHeader().getBank() == null ? 0 : -1;
                    }
                    else
                    {
                        ret = l1.getHeader().getBank().
                            compareTo(l2.getHeader().getBank());

                    }
                }

                return ret;
            }
            catch(IOException e)
            {
                throw new IllegalStateException(e);
            }
        }
    };

    /**
     * Collection of {@code LogicalFile} instances.
     * @serial
     */
    private List<LogicalFileModel> entities;

    /**
     * Gets the entities of the model.
     *
     * @return all entities of the model.
     */
    public List<LogicalFileModel> getEntities()
    {
        if(this.entities == null)
        {
            this.entities = Collections.synchronizedList(new ArrayList(50));
        }

        return this.entities;
    }

    protected void clear()
    {
        final int oldSize = this.getSize();
        this.getEntities().clear();
        this.fireIntervalRemoved(this, 0, oldSize);
    }

    protected void addAll(final Collection<LogicalFileModel> files)
    {
        final int oldSize = this.getSize();
        this.getEntities().addAll(files);
        Collections.sort(this.getEntities(), LOGICALFILE_SORTER);
        this.fireIntervalAdded(this, oldSize, this.getSize());
    }

    protected void remove(final int index)
    {
        this.getEntities().remove(index);
        this.fireIntervalRemoved(this, index, index);
    }

    protected void sort()
    {
        Collections.sort(this.getEntities(), LOGICALFILE_SORTER);
        fireContentsChanged(this, 0, this.getEntities().size());
    }

    //----------------------------------------------PhysicalFileFrameListModel--

}

/**
 * Extension to {@code Transaction} adding a property holding the index of the
 * transaction in a logical file.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3525 2007-05-16 12:05:28Z schulte2005 $
 */
class TransactionModel extends Transaction
{
    //--TransactionModel--------------------------------------------------------

    /**
     * Index of the transaction in the logical file.
     * @serial
     */
    private final int index;

    /**
     * Creates a new {@code TransactionModel} instance.
     *
     * @param index the index of {@code transaction} in a logical file.
     * @param transaction the data to initialize the new instance with.
     */
    public TransactionModel(final int index, final Transaction transaction)
    {
        super();
        this.index = index;
        this.setAmount(transaction.getAmount());
        this.setCurrency(transaction.getCurrency());
        this.setDescriptions(transaction.getDescriptions());
        this.setExecutiveAccount(transaction.getExecutiveAccount());
        this.setExecutiveBank(transaction.getExecutiveBank());
        this.setExecutiveExt(transaction.getExecutiveExt());
        this.setExecutiveName(transaction.getExecutiveName());
        this.setPrimaryBank(transaction.getPrimaryBank());
        this.setReference(transaction.getReference());
        this.setTargetAccount(transaction.getTargetAccount());
        this.setTargetBank(transaction.getTargetBank());
        this.setTargetExt(transaction.getTargetExt());
        this.setTargetName(transaction.getTargetName());
        this.setType(transaction.getType());
    }

    /**
     * Gets the index of the transaction in the logical file.
     *
     * @return the index of the transaction in the logical file.
     */
    public int getIndex()
    {
        return this.index;
    }

    //--------------------------------------------------------TransactionModel--
}

/**
 * Adapts a {@code LogicalFile} adding a property holding the index of the
 * file in a physical file.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileFrame.java 3525 2007-05-16 12:05:28Z schulte2005 $
 */
class LogicalFileModel implements LogicalFile
{
    //--LogicalFileModel--------------------------------------------------------

    /** Index of the logical in the physical file. */
    private final int index;

    /** {@code LogicalFile} delegate. */
    private final LogicalFile logicalFile;

    /**
     * Creates a new {@code LogicalFileModel} instance.
     *
     * @param index the index of {@code logicalFile} in a physical file.
     * @param logicalFile the data to initialize the new instance with.
     */
    public LogicalFileModel(final int index, final LogicalFile logicalFile)
    {
        super();
        this.index = index;
        this.logicalFile = logicalFile;
    }

    /**
     * Gets the index of the logical file in the physical file.
     *
     * @return the index of the logical file in the physical file.
     */
    public int getIndex()
    {
        return this.index;
    }

    //--------------------------------------------------------LogicalFileModel--
    //--LogicalFile-------------------------------------------------------------

    public Header getHeader() throws IOException
    {
        return this.logicalFile.getHeader();
    }

    public Header setHeader(final Header header) throws IOException
    {
        return this.logicalFile.setHeader(header);
    }

    public Checksum getChecksum() throws IOException
    {
        return this.logicalFile.getChecksum();
    }

    public void createTransaction(final Transaction transaction)
    throws IOException
    {
        this.logicalFile.createTransaction(transaction);
    }

    public Transaction getTransaction(final int index) throws IOException
    {
        return this.logicalFile.getTransaction(index);
    }

    public Transaction setTransaction(final int index,
        final Transaction transaction) throws IOException
    {
        return this.logicalFile.setTransaction(index, transaction);
    }

    public Transaction removeTransaction(final int index) throws IOException
    {
        return this.logicalFile.removeTransaction(index);
    }

    //-------------------------------------------------------------LogicalFile--
}
